#!/usr/bin/env python3
'''
Version file automatically generated by setuptools-scm via hatch-vcs.
https://pypi.org/project/setuptools-scm/
https://github.com/ofek/hatch-vcs
Do not track this file and do not edit it manually.
'''
VERSION = '2024.7'
VERSION_TUPLE = (2024, 7)
